# Import python libs
import os
import sys
import tempfile
import uuid
from typing import List

from dict_tools import data


def __init__(hub):
    """
    :param hub: The redistributed pop central hub.
    """
    hub.pop.sub.load_subdirs(hub.tiamat)
    os.environ["POP_BUILD"] = "1"
    os.environ["TIAMAT_BUILD"] = "1"
    hub.tiamat.BUILDS = {}
    hub.tiamat.SYSTEMD = ("rhel7", "rhel8", "arch", "debian10")
    hub.tiamat.SYSTEMD_DIR = "usr/lib/systemd/system/"
    hub.tiamat.SYSV = ("rhel5", "rhel6")
    hub.tiamat.SYSV_DIR = "etc/init.d"


def cli(hub):
    """
    Execute the routine from the CLI.

    :param hub: The redistributed pop central hub.
    """
    hub.pop.config.load(["tiamat"], cli="tiamat")
    if hub.SUBPARSER == "build":
        hub.tiamat.build.builder(
            name=hub.OPT.tiamat.name,
            requirements=hub.OPT.tiamat.requirements,
            sys_site=hub.OPT.tiamat.system_site,
            exclude=hub.OPT.tiamat.exclude,
            directory=os.path.abspath(hub.OPT.tiamat.directory),
            pyinstaller_version=hub.OPT.tiamat.pyinstaller_version,
            pyinstaller_runtime_tmpdir=hub.OPT.tiamat.pyinstaller_runtime_tmpdir,
            datas=hub.OPT.tiamat.datas,
            build=hub.OPT.tiamat.build,
            pkg=hub.OPT.tiamat.pkg,
            onedir=hub.OPT.tiamat.onedir,
            pyenv=hub.OPT.tiamat.pyenv,
            run=hub.OPT.tiamat.run,
            no_clean=hub.OPT.tiamat.no_clean,
            locale_utf8=hub.OPT.tiamat.locale_utf8,
            dependencies=hub.OPT.tiamat.dependencies,
            release=hub.OPT.tiamat.release,
            pkg_tgt=hub.OPT.tiamat.pkg_tgt,
            pkg_builder=hub.OPT.tiamat.pkg_builder,
            srcdir=hub.OPT.tiamat.srcdir,
            system_copy_in=hub.OPT.tiamat.system_copy_in,
            tgt_version=hub.OPT.tiamat.tgt_version,
            venv_plugin=hub.OPT.tiamat.venv_plugin,
            python_bin=hub.OPT.tiamat.python_bin,
            omit=hub.OPT.tiamat.omit,
            pyinstaller_args=hub.OPT.tiamat.pyinstaller_args,
        )
    elif hub.SUBPARSER == "clean":
        hub.tiamat.clean.all(hub.OPT.tiamat.directory)
    elif hub.SUBPARSER == "install":
        raise NotImplementedError()
    elif hub.SUBPARSER == "freeze":
        if getattr(sys, "frozen", False):
            freeze = os.path.join(sys._MEIPASS, "app_freeze.txt")
            with open(freeze) as fh:
                print(fh.read())
        else:
            raise OSError("freeze can only be called when tiamat is run as a binary")
    elif hub.SUBPARSER is None:
        hub.config.ARGS["parser"].print_help()
        sys.exit()


def new(
    hub,
    name: str,
    requirements: str,
    sys_site: bool,
    exclude: List[str],
    directory: str,
    pyinstaller_version: str = "4.0",
    pyinstaller_runtime_tmpdir: str = None,
    datas: set = None,
    build: str = None,
    pkg: str = None,
    onedir: bool = False,
    pyenv: str = "system",
    run: str = "run.py",
    locale_utf8: bool = False,
    dependencies: str = None,
    release: str = None,
    pkg_tgt: str = None,
    pkg_builder: str = None,
    srcdir: str = None,
    system_copy_in: List[str] = None,
    python_bin: str = None,
    omit: List[str] = None,
    pyinstaller_args: List[str] = None,
    venv_plugin: str = None,
) -> str:
    """
    :param hub: The redistributed pop central hub.
    :param name: The name of the project to build.
    :param requirements: The name of the requirements.txt file to use.
    :param sys_site: Include the system site-packages when building.
    :param exclude: A list of exclude files or python modules, these python packages will be ignored by pyinstaller
    :param directory: The path to the directory to build from.
        This denotes the root of the python project source tree to work from.
        This directory should have the setup.py and the paths referenced in configurations will assume that this is the root path they are working from.
    :param pyinstaller_version: The version of pyinstaller to use for packaging
    :param pyinstaller_runtime_tmpdir: Pyinstaller runtime tmpdir.
    :param datas: PyInstaller datas mapping
    :param build: Enter in commands to build a non-python binary into the deployed binary.
        The build options are set on a named project basis. This allows for multiple shared
        binaries to be embedded into the final build:

        build:
          libsodium:
            make:
              - wget libsodium
              - tar xvf libsodium*
              - cd libsodium
              - ./configure
              - make
            src: libsodium/libsodium.so
            dest: lib64/
    :param pkg: Options for building packages.
    :param onedir: Instead of producing a single binary produce a directory with all components.
    :param pyenv: The python version to build with.
    :param run: The location of the project run.py file.
    :param locale_utf8: Use the UTF-8 locale with PyInstaller, as in PEP538 and PEP540.
    :param dependencies: Comma separated list of dependencies needed for the build.
    :param release: Release string i.e. '1.el7'.
    :param pkg_tgt: The os/distribution target to build the package against.
    :param pkg_builder: The package builder plugin to use.
    :param srcdir: Install all of the python package sources and/or wheels found in this directory.
    :param system_copy_in: A list of directories to copy into the build venv that are not otherwise detected.
    :param python_bin: The python binary to use for system calls
    :param omit: Omit files by glob
    :param pyinstaller_args: Args to pass straight through to pyinstaller
    :param venv_plugin: The virtualenvironment plugin to use
    :return: The build name.
    """
    if name == "." or name is None:
        name = os.path.basename(os.path.abspath("."))
    venv_dir = tempfile.mkdtemp()
    is_win = os.name == "nt"
    if python_bin is None:
        if is_win:
            python_bin = os.path.join(venv_dir, "Scripts", "python")
        else:
            python_bin = os.path.join(venv_dir, "bin", "python3")
    if is_win:
        s_path = os.path.join(venv_dir, "Scripts", name)
    elif locale_utf8:
        s_path = "env PYTHONUTF8=1 LANG=POSIX " + os.path.join(venv_dir, "bin", name)
    else:
        s_path = os.path.join(venv_dir, "bin", name)

    if datas is None:
        datas = set()

    cmd = [python_bin, "-B", "-OO", "-m", "PyInstaller"]

    # Determine the python modules that should be excluded
    excluded = set()
    for ex in exclude:
        if os.path.isfile(ex):
            with open(ex) as exf:
                for line in exf:
                    line = line.strip()
                    if line:
                        excluded.add(line)
        else:
            excluded.add(ex)

    for ex in excluded:
        cmd.extend(["--exclude-module", ex])

    bname = str(uuid.uuid1())
    requirements = os.path.join(directory, requirements)
    hub.tiamat.BUILDS[bname] = data.NamespaceDict(
        name=name,
        build=build,
        pkg=pkg,
        pkg_tgt=pkg_tgt,
        pkg_builder=pkg_builder,
        dependencies=dependencies,
        release=release,
        binaries=[],
        is_win=is_win,
        exclude=excluded,
        requirements=requirements,
        sys_site=sys_site,
        dir=os.path.abspath(directory),
        srcdir=srcdir,
        pyinstaller_version=pyinstaller_version,
        pyinstaller_runtime_tmpdir=pyinstaller_runtime_tmpdir,
        system_copy_in=system_copy_in,
        run=os.path.join(directory, run),
        spec=os.path.join(directory, f"{name}.spec"),
        pybin=python_bin,
        s_path=s_path,
        venv_dir=venv_dir,
        vroot=os.path.join(venv_dir, "lib"),
        onedir=onedir,
        all_paths=set(),
        imports=set(),
        datas=datas,
        cmd=cmd,
        pyenv=pyenv,
        pypi_args=[
            s_path,
            "--log-level=INFO",
            "--noconfirm",
            "--onedir" if onedir else "--onefile",
            "--clean",
        ]
        + list(pyinstaller_args),
        locale_utf8=locale_utf8,
        omit=omit,
        venv_plugin=venv_plugin,
    )
    req = hub.tiamat.build.mk_requirements(bname)
    hub.tiamat.BUILDS[bname].req = req
    return bname



__func_alias__ = {"bin_": "bin"}


def bin_(hub, bname: str) -> str:
    """
    Ensure that the desired binary version is present and return the path to
    the python bin to call.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    pyvenv_bin = hub.tiamat.cmd.run(["which", "pyvenv"]).stdout.rstrip()
    hub.tiamat.cmd.run([pyvenv_bin, "--help"], fail_on_error=True)
    return pyvenv_bin


def create(hub, bname: str):
    """
    Make a virtual environment based on the version of python used to call this script.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]

    env_bin = hub.tiamat.virtualenv.pyvenv.bin(bname)
    cmd = [env_bin, opts.venv_dir, "--clear"]
    if opts.sys_site:
        cmd.append("--system-site-packages")

    hub.tiamat.cmd.run(cmd, fail_on_error=True)



def build(hub, bname: str):
    """
    Given the build arguments, Make a package!

    :param bname: The name of the build configuration to use from the build.conf file.
    """
    pkg_builder = hub.OPT.tiamat.pkg_builder
    build_func = getattr(hub.tiamat.pkg, pkg_builder).build
    build_func(bname)



# Import python libs
import os
import shutil


def report(hub, bname: str):
    """
    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    art = os.path.abspath(os.path.join("dist", opts.name))

    # This is the only print statement from the program, everything else gets logged through stderr
    print(art)

    # Save the location of the result to an environment variable for CICD or wrapper scripts
    os.environ["TIAMAT_BUILD_PACKAGE"] = art


def clean(hub, bname: str):
    """
    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    shutil.rmtree(opts.venv_dir)
    os.remove(opts.spec)
    os.remove(opts.req)
    try:
        # try to remove pyinstaller warn-*** file
        os.remove(os.path.join(opts.dir, "build", opts.name, f"warn-{opts.name}.txt"))
    except FileNotFoundError:
        pass



from typing import List


__func_alias__ = {"bin_": "bin"}


def env(hub, bname) -> List[str]:
    python_env = hub.tiamat.virtualenv.init.env(bname)
    if "env" not in python_env:
        python_env.append("env")
    return python_env + [
        'PYTHON_CONFIGURE_OPTS="--enable-shared --enable-ipv6"',
        'CONFIGURE_OPTS="--enable-shared --enable-ipv6"',
    ]


def bin_(hub, bname: str) -> List[str]:
    """
    Ensure that the desired binary version is present and return the path to
    the python bin to call.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    ret = hub.tiamat.cmd.run(["which", "venv"]).stdout.rstrip()
    if ret:
        venv_bin = [ret]
    else:
        venv_bin = [opts.pybin, "-m", "venv"]

    hub.tiamat.cmd.run(venv_bin + ["--help"], fail_on_error=True)
    return venv_bin


def create(hub, bname: str):
    """
    Make a virtual environment based on the version of python used to call this script.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]

    cmd = (
        hub.tiamat.virtualenv.venv.env(bname)
        + hub.tiamat.virtualenv.venv.bin(bname)
        + [opts.venv_dir, "--clear"]
    )
    if opts.sys_site:
        cmd.append("--system-site-packages")

    hub.tiamat.cmd.run(cmd, fail_on_error=True)



"""
The build plugin is used to execute the build routines for non-python components.
"""
# Import python libs
import glob
import os
import pprint
import shutil
import tempfile
from typing import List

import requests


def mk_requirements(hub, bname: str) -> str:
    """
    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    :return:
    """
    opts = hub.tiamat.BUILDS[bname]
    req = os.path.join(opts.dir, "__build_requirements.txt")
    with open(opts.requirements) as rfh:
        data_ = rfh.read()
    with open(req, "w+") as wfh:
        wfh.write(data_)
    return req


def builder(
    hub,
    name,
    requirements: str,
    sys_site: str,
    exclude: List[str],
    directory: str,
    pyinstaller_version: str = "4.0",
    pyinstaller_runtime_tmpdir: str = None,
    datas: set = None,
    build: str = None,
    pkg: str = None,
    onedir: bool = False,
    pyenv: str = "system",
    run: str = "run.py",
    no_clean: bool = False,
    locale_utf8: bool = False,
    dependencies: str = None,
    release: str = None,
    pkg_tgt: str = None,
    pkg_builder: str = None,
    srcdir: str = None,
    system_copy_in: List[str] = None,
    tgt_version: str = None,
    python_bin: str = None,
    omit: List[str] = None,
    venv_plugin: str = None,
    pyinstaller_args: List[str] = None,
):
    """
    :param hub: The redistributed pop central hub.
    :param name: The name of the project to build.
    :param requirements: The name of the requirements.txt file to use.
    :param sys_site: Include the system site-packages when building.
    :param exclude: A list of exclude files or python modules, these python packages will be ignored by pyinstaller
    :param directory: The path to the directory to build from.
        This denotes the root of the python project source tree to work from.
        This directory should have the setup.py and the paths referenced in configurations will assume that this is the root path they are working from.
    :param pyinstaller_version: The version of pyinstaller to use for packaging
    :param pyinstaller_runtime_tmpdir: Pyinstaller runtime tmpdir.
    :param build: Enter in commands to build a non-python binary into the deployed binary.
        The build options are set on a named project basis. This allows for multiple shared
        binaries to be embedded into the final build:

        build:
          libsodium:
            make:
              - wget libsodium
              - tar xvf libsodium*
              - cd libsodium
              - ./configure
              - make
            src: libsodium/libsodium.so
            dest: lib64/
    :param pkg: Options for building packages.
    :param onedir: Instead of producing a single binary produce a directory with all components.
    :param pyenv: The python version to build with.
    :param run: The location of the project run.py file.
    :param no_clean: Don't run the clean up sequence, this will leave the venv, spec file and other artifacts.
    :param locale_utf8: Use the UTF-8 locale with PyInstaller, as in PEP538 and PEP540.
    :param dependencies: Comma separated list of dependencies needed for the build.
    :param release: Release string i.e. '1.el7'.
    :param pkg_tgt: The os/distribution target to build the package against.
    :param pkg_builder: The package builder plugin to use.
    :param srcdir: Install all of the python package sources and/or wheels found in this directory.
    :param system_copy_in: A list of directories to copy into the build venv that are not otherwise detected.
    :param tgt_version: Target package version.
    :param python_bin: The python binary to use for system calls
    :param pyinstaller_args: Args to pass straight through to pyinstaller
    :param venv_plugin: The virtual environment plugin to use
    """

    bname = hub.tiamat.init.new(
        name=name,
        requirements=requirements,
        sys_site=sys_site,
        exclude=exclude or [],
        directory=directory,
        pyinstaller_version=pyinstaller_version,
        pyinstaller_runtime_tmpdir=pyinstaller_runtime_tmpdir,
        datas=datas,
        build=build,
        pkg=pkg,
        onedir=onedir,
        pyenv=pyenv,
        run=run,
        locale_utf8=locale_utf8,
        dependencies=dependencies,
        release=release,
        pkg_tgt=pkg_tgt,
        pkg_builder=pkg_builder,
        srcdir=srcdir,
        system_copy_in=system_copy_in,
        python_bin=python_bin,
        omit=omit,
        pyinstaller_args=pyinstaller_args or [],
        venv_plugin=venv_plugin,
    )
    hub.log.debug(
        f"Build config '{bname}':\n{pprint.pformat(hub.tiamat.BUILDS[bname])}"
    )
    hub.tiamat.virtualenv.init.create(bname)
    if tgt_version:
        hub.tiamat.BUILDS[bname].version = tgt_version
    else:
        hub.tiamat.data.version(bname)
    hub.tiamat.build.make(bname)
    hub.tiamat.virtualenv.init.scan(bname)
    hub.tiamat.virtualenv.init.mk_adds(bname)
    hub.tiamat.inst.mk_spec(bname)
    hub.tiamat.inst.call(bname)
    if pkg_tgt:
        hub.tiamat.pkg.init.build(bname)
    hub.tiamat.post.report(bname)
    if not no_clean:
        hub.tiamat.post.clean(bname)


def make(hub, bname: str):
    """
    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    build = opts.build
    if not build:
        return
    bdir = tempfile.mkdtemp()
    cur_dir = os.getcwd()
    os.chdir(bdir)
    if opts.srcdir:
        for fn in os.listdir(opts.srcdir):
            shutil.copy(os.path.join(opts.srcdir, fn), bdir)
    for proj, conf in build.items():
        if not opts.srcdir:
            if "sources" in conf:
                sources = conf["sources"]
                if isinstance(sources, str):
                    sources = [sources]
                for source in sources:
                    response = requests.get(source)
                    with open(
                        os.path.join(bdir, os.path.split(source)[1]), "wb"
                    ) as file:
                        file.write(response.content)
        if "make" in conf:
            for cmd in conf["make"]:
                hub.tiamat.cmd.run(cmd, shell=True, cwd=bdir, fail_on_error=True)
        if "src" in conf and "dest" in conf:
            srcs = conf["src"]
            dest = os.path.join(opts.venv_dir, conf["dest"])
            hub.log.info(f"Copying: {srcs}->{dest}")
            if not isinstance(srcs, (list, tuple)):
                srcs = [srcs]
            final_srcs = set()
            for src in srcs:
                globed = glob.glob(src)
                if not globed:
                    hub.log.warning(f"Expression f{src} does not match any file paths")
                    continue
                final_srcs.update(globed)
            for src in final_srcs:
                fsrc = os.path.join(bdir, src)
                if os.path.isfile(fsrc):
                    try:
                        shutil.copy(fsrc, dest, follow_symlinks=True)
                    except OSError as e:
                        hub.log.warning(f"Unable to copy file {fsrc} to {dest}: {e}")
                    hub.tiamat.BUILDS[bname].binaries.append(
                        (os.path.join(dest, os.path.basename(fsrc)), ".")
                    )
                elif os.path.isdir(fsrc):
                    shutil.copytree(fsrc, dest)
                else:
                    hub.log.warning(f"FAILED TO FIND FILE {fsrc}")
    os.chdir(cur_dir)



import subprocess
from typing import Any
from typing import Dict
from typing import List

from dict_tools import data

__virtualname__ = "cmd"


def _log_output(hub, stdout: str, stderr: str):
    for line in stdout.splitlines():
        hub.log.info(line)

    for line in stderr.splitlines():
        hub.log.debug(line)


def run(
    hub,
    cmd: List[str],
    shell: bool = False,
    timeout: int = None,
    fail_on_error: bool = False,
    **kwargs,
) -> Dict[str, Any]:
    """
    Run every command the same way
    :param hub:
    :param cmd:
    :param shell:
    :param timeout:
    :param fail_on_error:
    :return:
    """
    if shell is True and not isinstance(cmd, str):
        cmd = " ".join(cmd)
    elif shell is False and isinstance(cmd, str):
        cmd = cmd.split()

    proc = subprocess.Popen(
        cmd, shell=shell, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs
    )

    retcode = proc.wait(timeout=timeout)

    stdout, stderr = proc.communicate(timeout=timeout)
    if isinstance(stdout, bytes):
        stdout = stdout.decode()
    if isinstance(stderr, bytes):
        stderr = stderr.decode()

    _log_output(hub, stdout, stderr)

    if retcode and fail_on_error:
        raise ChildProcessError(stderr)

    return data.NamespaceDict(
        retcode=retcode, stdout=stdout, stderr=stderr, pid=proc.pid
    )



from typing import List

__func_alias__ = {"bin_": "bin"}


def bin_(hub, bname: str) -> List[str]:
    """
    Ensure that the desired binary version is present and return the path to
    the python bin to call.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    ret = hub.tiamat.cmd.run(["which", "virtualenv3"]).stdout.rstrip()
    if ret:
        virtualenv_bin = [ret]
    else:
        virtualenv_bin = [opts.pybin, "-m", "virtualenv", "--verbose"]

    hub.tiamat.cmd.run(virtualenv_bin + ["--help"], fail_on_error=True)
    return virtualenv_bin


def create(hub, bname: str):
    """
    Make a virtual environment based on the version of python used to call this script.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]

    env_bin = hub.tiamat.virtualenv.virtualenv.bin(bname)
    cmd = env_bin + [opts.venv_dir, "--clear"]
    pyenv = opts.get("pyenv")
    if pyenv and pyenv != "system":
        cmd.extend(["--python", opts.pyenv])
    if opts.sys_site:
        cmd.append("--system-site-packages")

    hub.tiamat.cmd.run(cmd, fail_on_error=True)



import logging
import os
import shutil
from typing import Callable
from typing import Tuple

log = logging.getLogger(__name__)
__func_alias__ = {"all_": "all"}


def _report_error(_, path: str, error: Tuple[Callable, Exception]):
    if error[0] is FileNotFoundError:
        log.debug(f"File {path} is already deleted")
    else:
        log.error(error[1])


def remove_dist(hub, directory: str):
    """
    Clean the resulting dist dir

    This is in it's own function in case one day someone adds the --distpath option to the pyinstaller call
    """
    dist_dir = os.path.join(directory, "dist")
    hub.log.info(f"Removing {dist_dir}")
    shutil.rmtree(dist_dir, onerror=_report_error)


def remove_build(hub, directory: str):
    """
    Clean the build dir
    This is in it's own function in case one day someone adds the --workpath option to the pyinstaller call
    """
    build_dir = os.path.join(directory, "build")
    hub.log.info(f"Removing {build_dir}")
    shutil.rmtree(os.path.join(directory, "build"), onerror=_report_error)


def remove_pytest_cache(hub, directory: str):
    """
    Remove the pytest cache
    """
    pycache_dir = os.path.join(directory, ".pytest_cache")
    hub.log.info(f"Removing {pycache_dir}")
    shutil.rmtree(pycache_dir, onerror=_report_error)


def remove_logs(hub, directory: str):
    """
    Recursively remove all files ending in .log from the build directory
    """
    for parent, _, files in os.walk(directory):
        for f in files:
            if f.endswith(".log"):
                log_file = os.path.join(parent, f)
                hub.log.info(f"Removing {log_file}")
                try:
                    os.remove(log_file)
                except Exception as e:
                    _report_error(_, log_file, (type(e), e))


def all_(hub, directory: str):
    """
    Run all the functions in this plugin
    """
    for func in hub.tiamat.clean:
        if func.__name__ == "all":
            # Skip this function or we would have an infinite loop
            continue

        func(directory)



# Import python libs
import os


def version(hub, bname: str):
    """
    Gather the version number of the pop project if possible.

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    dir_ = hub.tiamat.BUILDS[bname].dir
    name = hub.tiamat.BUILDS[bname].name
    name = name.replace("-", "_")
    path = os.path.join(dir_, name, "version.py")
    _locals = {}
    version_ = "1"
    try:
        if os.path.isfile(path):
            with open(path) as fp:
                exec(fp.read(), None, _locals)
                version_ = _locals["version"]
    except Exception:
        pass
    hub.tiamat.BUILDS[bname].version = version_



"""
Use the system python interpreter and don't create an environment
"""
import os
import sys
from typing import List

__func_alias__ = {"bin_": "bin"}


def bin_(hub, bname: str) -> List[str]:
    """

    :param hub:
    :param bname:
    :return:
    """
    opts = hub.tiamat.BUILDS[bname]
    return opts.pybin or sys.executable


def create(hub, bname: str):
    """
    Change up the opts for this build so that they use the appropriate system python directories

    :param hub: The redistributed pop central hub.
    :param bname: The name of the build configuration to use from the build.conf file.
    """
    opts = hub.tiamat.BUILDS[bname]
    pybin = hub.tiamat.virtualenv.system.bin(bname)

    # Prepare "virtual" environment by symlinking real binaries to that env
    bin_path = os.path.join(opts.venv_dir, "bin")
    os.makedirs(bin_path, exist_ok=True)
    python_path = os.path.join(bin_path, "python")
    python3_path = os.path.join(bin_path, "python3")

    # Symlink the environments pybin with the virtual environment
    os.symlink(pybin, python_path)
    os.symlink(pybin, python3_path)
